//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Error/Sys.h,v $
// $Id: Sys.h,v 1.7 2004/04/20 22:27:08 ah Exp $
//
// Description: error class for system errors
//
//---------------------------------------------------------------------------

#ifndef AH_ERROR_SYS_H		// prevent multiple includes
#define AH_ERROR_SYS_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Error/Basic.h>
#include <cerrno>		// to know errno
#include <string>		// to know strerror

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/// system error class
/**
   ErrorSys is used for all system errors which in C are usually 
   handled with errno and strerror(). ErrorSys encapsulates this.

   Please include <Error/Sys.h>.
*/
class ErrorSys: public ErrorBasic
{
  public:
    /// default constructor
    ErrorSys()
	: ErrorBasic(strerror(errno)), errorCode(errno) {}
    /// constructor
    ErrorSys(int code)
	: ErrorBasic(strerror(code)), errorCode(code) {}
    /// constructor
    ErrorSys(const string& m)
	: ErrorBasic(m + string(": ") + strerror(errno)), errorCode(errno) {}
    /// constructor
    ErrorSys(const string& m, int code)
	: ErrorBasic(m + string(": ") + strerror(code)), errorCode(code) {}
    /// copy constructor
    ErrorSys(const ErrorSys& src)
	: ErrorBasic(src), errorCode(src.errorCode) {}
    /// assignment operator:
    const ErrorSys& operator=(const ErrorSys &src)
	{
	    if (&src != this)
	    {
		ErrorBasic::operator=(src);
		errorCode = src.errorCode;
	    }
	    return *this;
	}
    /// destructor:
    virtual ~ErrorSys () throw() {}

    /// allow to ask for the system error code and its related 
    // message:
    int	getErrorCode() const
	{ return errorCode; }

  protected:
    int errorCode;		// system error code
};


// common used system errors:
/// system error (out of memory)
class ErrorSysOutOfMem: public ErrorSys
{
  public:
    /// default constructor
    ErrorSysOutOfMem()
	: ErrorSys(ENOMEM) {}
    /// constructor
    ErrorSysOutOfMem(const string& m)
	: ErrorSys(m, ENOMEM) {}
    /// copy constructor
    ErrorSysOutOfMem(const ErrorSys& src)
	: ErrorSys(src) {}
    /// assignment operator:
    const ErrorSysOutOfMem& operator=(const ErrorSysOutOfMem &src)
	{ if (&src != this) ErrorSys::operator=(src); return *this; }
    /// destructor:
    virtual ~ErrorSysOutOfMem () throw () {}
};

END_NAMESPACE

#endif // ERROR_SYS_H
//**********************************************************************
//*			     E N D   O F   F I L E                     *
//**********************************************************************






