//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Speaker.h,v $
// $Id: Speaker.h,v 1.7 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_SPEAKER_H	// prevent multiple includes
#define ETECH_SPEAKER_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Types.h>
#include <AH/Etech/Impedance.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents a pure speaker system
/**
 {\tt Speaker} represents a real speaker system.
 */
class Speaker
{
  private:
    const char*	name;

  protected:
    Dbl		Rdc;	// DC resistance of coil
    Dbl		Ls;	// inductivity of coil
    Dbl		Fr;	// free resonance frequency
    Dbl		Qm;	// mechanical damping factor
    Dbl		Qe;	// electrical damping factor
    Dbl		Qt;	// overall damping factor
    Dbl		Va;	// equivalent volume
    Dbl		Spl;	// speaker sensitivity dB/W/m

  public:
    /// default constructor
    Speaker();

    /// copy constructor
    Speaker(const Speaker& s);

    /// constructor
    Speaker(const char* n);

    /// virtual destructor
    virtual ~Speaker() {}


    /// get name of speaker
    const char* getName() const { return name; }

    /// get overall damping factor
    Dbl getQt() const { return Qt; }

    /// get electrical damping factor
    Dbl getQe() const { return Qe; }

    /// get mechanical damping factor
    Dbl getQm() const { return Qm; }

    /// get resonance frequency
    Dbl getFres() const { return Fr; }

    /// get equivalent volume
    Dbl getVa() const { return Va; }

    /// get acoustic power level dB/W/m at specified frequency
    virtual Dbl getSpl(Dbl freq) const;

    /// get impedance of voice coil at specified frequency
    virtual Impedance getImp(Dbl freq) const;

    /// assignment operator
    const Speaker& operator = (const Speaker& s);

}; // class Speaker


END_NAMESPACE

#endif // ETECH_SPEAKER_H
