//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Types.h,v $
// $Id: Types.h,v 1.10 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_TYPES_H	// prevent multiple includes
#define ETECH_TYPES_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <complex>

// --- forward declarations ---

class ostream;

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// declaration of a scalar number
/**
 {\tt Dbl} is a scalar number, which is almost everwhere used in the
 {\em Etech} library. The reason for using this definition is to make
 it easier using floating point types other than {\tt double} simply
 by changing this definition.
 */
typedef double Dbl;

/// declaration of a complex number
/**
 {\tt Cplx} is a complex number, which is almost everwhere used in the
 {\em Etech} library. The reason for using this definition is to make
 it easier using complex types other than {\tt double\_complex} simply
 by changing this definition.
 */
class Cplx
{
  protected:

    double_complex		 value;

  public:
    enum State {Valid, Nul, Inf} state;

    /// constructor
    Cplx (const Dbl re = 0, const State s = Valid)
	: value(re), state(s) {}
    /// constructor
    Cplx (const Dbl re, const Dbl im, const State s = Valid)
	: value(re, im), state(s) {}
    /// constructor
    Cplx (const double_complex &c, const State s = Valid)
	: value(c), state(s) {}
    /// copy constructor
    Cplx (const Cplx& c) 
	: value(c.value), state(c.state) {}

    /// return {\tt true} if value is infinity
    bool isInf() const { return (Inf == state); }
    /// return {\tt true} if value is null
    bool isNul() const { return (Nul == state); }
    /// return {\tt true} if value is valid (neither null nor infinity)
    bool isValid() const { return (Valid == state); }

    /// overloaded operator for addition with a {\tt Cplx} object
    Cplx& operator += (const Cplx& c);
    /// overloaded operator for addition with a {\tt Dbl} object
    Cplx& operator += (const Dbl& d);

    /// overloaded operator for subtraktion with a {\tt Cplx} object
    Cplx& operator -= (const Cplx& c);
    /// overloaded operator for subtraktion with a {\tt Dbl} object
    Cplx& operator -= (const Dbl& d);

    /// overloaded operator for multiplication with a {\tt Cplx} object
    Cplx& operator *= (const Cplx& c);
    /// overloaded operator for multiplication with a {\tt Dbl} object
    Cplx& operator *= (const Dbl& d);

    /// overloaded operator for division with a {\tt Cplx} object
    Cplx& operator /= (const Cplx& c);
    /// overloaded operator for division with a {\tt Dbl} object
    Cplx& operator /= (const Dbl& d);

    /// overloaded operator for unary minus of a {\tt Cplx} object
    Cplx operator - () const;

    /// overloaded operator for output into streams
    friend ostream& operator << (ostream &out, const Cplx &c);

    /// overloaded operator for addition of two {\tt Cplx} objects
    friend Cplx operator + (const Cplx& c1,
			    const Cplx& c2);
    /// overloaded operator for addition of {\tt Cplx} + {\tt Cplx}
    friend Cplx operator + (const Cplx& c,
			    const Dbl &d);
    /// overloaded operator for addition of {\tt Cplx} + {\tt Cplx}
    friend Cplx operator + (const Dbl &d,
			    const Cplx& c);

    /// overloaded operator for subtraction of two {\tt Cplx} objects
    friend Cplx operator - (const Cplx& c1,
			    const Cplx& c2);
    /// overloaded operator for subtraction of {\tt Cplx} - {\tt Cplx}
    friend Cplx operator - (const Cplx& c,
			    const Dbl& d);
    /// overloaded operator for subtraction of {\tt Cplx} - {\tt Cplx}
    friend Cplx operator - (const Dbl& d,
			    const Cplx& c);

    /// overloaded operator for multiplication of two {\tt Cplx} objects
    friend Cplx operator * (const Cplx& c1,
			    const Cplx& c2);
    /// overloaded operator for multiplication of {\tt Cplx} * {\tt Cplx}
    friend Cplx operator * (const Cplx& c,
			    const Dbl& d);
    /// overloaded operator for multiplication of {\tt Cplx} * {\tt Cplx}
    friend Cplx operator * (const Dbl& d,
			    const Cplx& c);

    /// overloaded operator for division of two {\tt Cplx} objects
    friend Cplx operator / (const Cplx& c1,
			    const Cplx& c2);
    /// overloaded operator for division of {\tt Cplx} / {\tt Cplx}
    friend Cplx operator / (const Cplx& c,
			    const Dbl& d);
    /// overloaded operator for division of {\tt Cplx} / {\tt Cplx}
    friend Cplx operator / (const Dbl& d,
			    const Cplx& c);

    /// overloaded operator == for comparison of two {\tt Cplx}
    friend bool operator == (const Cplx &c1, const Cplx& c2);
    /// overloaded operator != for comparison of two {\tt Cplx}
    friend bool operator != (const Cplx &c1, const Cplx& c2);
    /// overloaded operator < for comparison of two {\tt Cplx}
    friend bool operator < (const Cplx &c1, const Cplx& c2);
    /// overloaded operator > for comparison of two {\tt Cplx}
    friend bool operator > (const Cplx &c1, const Cplx& c2);

    /// return absolute of value of a {\tt Cplx} object
    friend Dbl abs (const Cplx &c) { return abs(c.value); }
    /// return argument of value of a {\tt Cplx} object
    friend Dbl arg (const Cplx &c) { return arg(c.value); }

};


END_NAMESPACE

#endif // ETECH_TYPES_H
