//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Profile.h,v $
// $Id: 
//
// Description: declaration of class AH::IOProfile
//
//---------------------------------------------------------------------------

#ifndef AH_IO_PROFILE_H		// prevent multiple includes
#define AH_IO_PROFILE_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/TimeStamp.h>
USING_NAMESPACE(AH)

#include <sys/times.h>
//#include <ctime>
#include <string>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)


/** profiling utility class. */
class IOProfile
{
 public:
    /** default constructor. */
    IOProfile() 
	: atStart(), atLastCatch(atStart) {}

    /** catch current time stamp. */
    void catchNow() { atLastCatch.update(); }

    /** return string containing elapsed time. */    
    std::string getLastRunTime() const
	{ IOTimeStamp now; return (now - atLastCatch).getTimeString(); }

    /** return string containing elapsed user time. */    
    std::string getLastUserRunTime() const
	{ IOTimeStamp now; return (now - atLastCatch).getUserTimeString(); }

    /** return string containing elapsed system time. */    
    std::string getLastSystemRunTime() const
	{ IOTimeStamp now; return (now - atLastCatch).getSystemTimeString(); }

    /** return string containing elapsed time. */    
    std::string getRunTime() const
	{ IOTimeStamp now; return (now - atStart).getTimeString(); }

    /** return string containing elapsed user time. */    
    std::string getUserRunTime() const
	{ IOTimeStamp now; return (now - atStart).getUserTimeString(); }

    /** return string containing elapsed system time. */    
    std::string getSystemRunTime() const
	{ IOTimeStamp now; return (now - atStart).getSystemTimeString(); }

 private:
    IOTimeStamp atStart;
    IOTimeStamp atLastCatch;
};	// IOProfile


END_NAMESPACE

#endif // !AH_IO_PROFILE_H

