//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Math/DataStat.h,v $
// $Id: DataStat.h,v 1.2 2004/04/06 23:14:43 ah Exp $ 
//
// Description: Interface to data statistic functions
//
//---------------------------------------------------------------------------

#ifndef AH_MATH_DATA_STAT_H
#define AH_MATH_DATA_STAT_H

// --- necessary includes ---

#include <AH/Config/Platform.h>

// --- types and classes ---

BEGIN_NAMESPACE(AH)


class DataStat
{
 public:
    /// calculate mean value of data array
    /**
       DataStat::calcMean() calculates <i>dataNum</i> double values from
       <i>data</i> and stores the mean value in <i>outMean</i>.
     */
    static void calcMean
    (
	/// input data
	const double* data,
	/// number of input data
	unsigned int dataNum,
	/// for mean value
	double* outMean
    );

    /// calculate max and minimum value of data array
    /**
       DataStat::calcMaxMin() calculates <i>dataNum</i> double values from
       <i>data</i> and stores the maximum value in <i>outMax</i> and the
       minimum value in <i>outMin</i>, if these are not NULL pointers. 
     */
    static void calcMaxMin
    (
	/// input data
	const double* data,
	/// number of input data
	unsigned int dataNum,
	/// for maximum value, may be 0 if not needed
	double* outMax,
	/// for minimum value, may be 0 if not needed
	double* outMin
    );

    /// calculate mean, max and minimum value of data array
    /**
       DataStat::calcMeanMaxMin() calculates <i>dataNum</i> double values from
       <i>data</i> and stores the mean value in <i>outMean</i>, the maximum
       value in <i>outMax</i> and the minimum value in <i>outMin</i>,
       if these are not NULL pointers. 
     */
    static void calcMeanMaxMin
    (
	/// input data
	const double* data,
	/// number of input data
	unsigned int dataNum,
	/// for mean value, may be 0 if not needed
	double* outMean,
	/// for maximum value, may be 0 if not needed
	double* outMax,
	/// for minimum value, may be 0 if not needed
	double* outMin
    );


 private:
};


// --- function prototypes ---

END_NAMESPACE
#endif // !AH_MATH_DATA_STAT_H
// --- end of file ---
