//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/OpAmp.c,v $
// $Id: OpAmp.c,v 1.10 2001/10/05 23:44:26 ah Exp $
//
// Description: implementation of class OpAmp
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: OpAmp.c,v 1.10 2001/10/05 23:44:26 ah Exp $ (c) 1997 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/OpAmp.h>
#include <AH/Etech/Resistor.h>
#include <AH/Etech/Capacitor.h>

// --- local stuff ---

// --- class member implementation ---


// default constructor
OpAmp::OpAmp() : 
    Zin(0), f(-1), Af(0), Name("Idealer OP"), A0(-1), Fg(-1), En(0), FgEn(-1), 
    In(0), FgIn(-1), Rout(0), Rin(0), Cin(0)
{
}



// simple constructor
OpAmp::OpAmp(const char* name, Dbl a0, Dbl ft) : 
    Zin(0), f(-1), Af(0), Name(name), A0(a0), Fg(ft/a0), En(0), FgEn(-1), 
    In(0), FgIn(-1), Rout(0) , Rin(0), Cin(0)
{
}



// complete constructor
OpAmp::OpAmp(const char* name, Dbl a0, Dbl ft, Dbl rout, Dbl en, 
	  Dbl fgen, Dbl in, Dbl fgin) : 
	Zin(0), f(-1), Af(0), Name(name), A0(a0), Fg(ft/a0), 
	En(en), FgEn(fgen), In(in), FgIn(fgin) , Rin(0), Cin(0)
{
    Rout = new Resistor(rout);
}



// copy constructor
OpAmp::OpAmp(const OpAmp& op) : 
    Zin(op.Zin), f(op.f), Af(op.Af), Name(op.Name), A0(op.A0), Fg(op.Fg), 
    En(op.En), FgEn(op.FgEn), In(op.In), FgIn(op.FgIn), Rout(0) , Rin(0), 
    Cin(0)
{
    if (op.Rout) Rout = new Resistor(*op.Rout);
    if (op.Rin)  Rin  = new Resistor(*op.Rin);
    if (op.Cin)  Cin  = new Capacitor(*op.Cin);
}



// destructor
OpAmp::~OpAmp() 
{
    if (Rout) delete Rout;
    if (Rin) delete Rin;
    if (Cin) delete Cin;
}



// get open loop input impedance at frequency 'freq':
const Impedance* OpAmp::getZin(Dbl freq)
{
    if (!Rin && !Cin) return 0;

    if (Cin)
    {
	if (Rin) Zin = *Rin + (*Cin)(freq);
	else     Zin = (*Cin)(freq);
    }
    else Zin = *Rin;

    return (&Zin);
}


// get open loop output impedance at frequency 'freq'
const Impedance* OpAmp::getZout(Dbl freq) 
{
    return Rout; 
}



// get differential open loop amplification at frequency 'freq':
const Cplx* OpAmp::getAd(Dbl freq)
{
    if ((0 > A0) || (0 > Fg)) return 0;

    if (freq != f)
    {
	f = freq;
	Cplx n(1, f/Fg);
	Af = A0/n;
    }

    return (&Af);
}


// --- overloaded operators ---

// assign operator
const OpAmp& OpAmp::operator = (const OpAmp& op)
{
    if (&op != this)
    {
	f = op.f;
	Af = op.Af;
	Name = op.Name;
	A0 = op.A0;
	Fg = op.Fg;
	En = op.En;
	FgEn = op.FgEn;
	In = op.In;
	FgIn = op.FgIn;

	if (op.Rout)
	{
	    if (Rout)	*Rout = *op.Rout;
	    else	Rout = new Resistor(*op.Rout);
	}
	else if (Rout)	delete Rout;

	if (op.Rin)
	{
	    if (Rin)	*Rin = *op.Rin;
	    else	Rin = new Resistor(*op.Rin);
	}
	else if (Rin)	delete Rin;

	if (op.Cin)
	{
	    if (Cin)	*Cin = *op.Cin;
	    else	Cin = new Capacitor(*op.Cin);
	}
	else if (Cin)	delete Cin;
    }
    return *this;
}

