//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/Types.c,v $
// $Id: Types.c,v 1.8 2004/04/20 22:14:57 ah Exp $
//
// Description: implementation of class Cplx
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: Types.c,v 1.8 2004/04/20 22:14:57 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Types.h>
#include <iostream>

BEGIN_NAMESPACE(Ah)

// --- local stuff ---

// --- class member implementation ---

// --- overloaded operators ---

//----------------------
// operators for output:
//----------------------

// overloaded operator for output into streams
std::ostream& operator << (std::ostream &out, const Cplx &c)
{
    if (true /*out.opfx()*/)
    {
	if (c.isInf())		out << "<inf>";
	else if (c.isNul())	out << 0;
	else			out << c.value;
    }
    return out; 
}

//------------------------
// operators for addition:
//------------------------

// overloaded operator for addition with a {\tt Cplx} object
Cplx& Cplx::operator += (const Cplx& c)
{
    if (isInf())	return *this;	// add with infinity = infinity
    if (c.isInf())	state = Inf;	// add with infinity = infinity
    else if (isNul())	value = c.value;// add with 0
    else if (c.isNul()) return *this;	// add with 0
    else		value += c.value; 
    return *this;
}

// overloaded operator for addition with a {\tt Dbl} object
Cplx& Cplx::operator += (const Dbl& d)
{
    if (isInf())	return *this;	// add with infinity = infinity
    else if (isNul())	value = d;	// add with 0
    else		value += d; 
    return *this;
}

//---------------------------
// operators for subtraction:
//---------------------------

// overloaded operator for subtraction with a {\tt Cplx} object
Cplx& Cplx::operator -= (const Cplx& c)
{
    if (isInf())	return *this;	// add with infinity = infinity
    if (c.isInf())	state = Inf;	// add with infinity = infinity
    else if (isNul())	value = -c.value;// add with 0
    else if (c.isNul()) return *this;	// add with 0
    else		value -= c.value; 
    return *this;
}

// overloaded operator for subtraction with a {\tt Dbl} object
Cplx& Cplx::operator -= (const Dbl& d)
{
    if (isInf())	return *this;	// add with infinity = infinity
    else if (isNul())	value = -d;	// add with 0
    else		value -= d; 
    return *this;
}

//------------------------------
// operators for multiplication:
//------------------------------

// overloaded operator for multiplication with a {\tt Cplx} object
Cplx& Cplx::operator *= (const Cplx& c)
{
    if (isInf()) return *this;		// mul with infinity = infinity
    if (isNul()) return *this;		// mul with 0 = 0
    if (c.isInf()) 	state = Inf;	// mul with infinity = infinity
    else if (c.isNul()) state = Nul;	// mul with 0 = 0
    else		value *= c.value; 
    return *this;
}

// overloaded operator for multiplication with a {\tt Dbl} object
Cplx& Cplx::operator *= (const Dbl& d)
{
    if (isInf()) return *this;		// mul with infinity = infinity
    if (isNul()) return *this;		// mul with 0 = 0
    value *= d; 
    return *this;
}

//------------------------
// operators for division:
//------------------------

// overloaded operator for division with a {\tt Cplx} object
Cplx& Cplx::operator /= (const Cplx& c)
{
    if (isInf()) return *this;		// infinity / x = infinity
    if (isNul()) return *this;		// 0 / x = 0
    if (c.isInf()) 	state = Nul;	// x / infinity = 0
    else if (c.isNul()) state = Inf;	// x / 0 = infinity
    else		value /= c.value; 
    return *this;
}

// overloaded operator for division with a {\tt Dbl} object
Cplx& Cplx::operator /= (const Dbl& d)
{
    if (isInf()) return *this;		// infinity / x = infinity
    if (isNul()) return *this;		// 0 / x = 0
    value /= d; 
    return *this;
}

// overloaded operator for unary minus of a {\tt Cplx} object
Cplx Cplx::operator - () const
{
    return Cplx(-value, state);
}


//-------------------------------
// friend operators for addition:
//-------------------------------

// overloaded operator for addition of two {\tt Cplx} objects
Cplx operator + (const Cplx& c1, const Cplx& c2)
{
    Cplx tmp(c1);
    return (tmp += c2);
}

// overloaded operator for addition of {\tt Cplx} + {\tt Cplx}
Cplx operator + (const Cplx& c, const Dbl &d)
{
    Cplx tmp(c);
    return (tmp += d);
}

// overloaded operator for addition of {\tt Cplx} + {\tt Cplx}
Cplx operator + (const Dbl& d, const Cplx& c)
{
    Cplx tmp(c);
    return (tmp += d);
}

//----------------------------------
// friend operators for subtraction:
//----------------------------------

// overloaded operator for subtraction of two {\tt Cplx} objects
Cplx operator - (const Cplx& c1, const Cplx& c2)
{
    Cplx tmp(c1);
    return (tmp -= c2);
}

// overloaded operator for subtraction of {\tt Cplx} - {\tt Cplx}
Cplx operator - (const Cplx& c, const Dbl& d)
{
    Cplx tmp(c);
    return (tmp -= d);
}

// overloaded operator for subtraction of {\tt Cplx} - {\tt Cplx}
Cplx operator - (const Dbl& d, const Cplx& c)
{
    if (c.isInf()) return c;	// sub with infinity = infinity
    if (c.isNul()) return d;	// sub with 0
    return (d - c.value); 
}

//-------------------------------------
// friend operators for multiplication:
//-------------------------------------

// overloaded operator for multiplication of two {\tt Cplx} objects
Cplx operator * (const Cplx& c1, const Cplx& c2)
{
    Cplx tmp(c1);
    return (tmp *= c2);
}

// overloaded operator for multiplication of {\tt Cplx} * {\tt Cplx}
Cplx operator * (const Cplx& c, const Dbl& d)
{
    Cplx tmp(c);
    return (tmp *= d);
}

// overloaded operator for multiplication of {\tt Cplx} * {\tt Cplx}
Cplx operator * (const Dbl& d, const Cplx& c)
{
    Cplx tmp(c);
    return (tmp *= d);
}

//-------------------------------
// friend operators for division:
//-------------------------------

// overloaded operator for division of two {\tt Cplx} objects
Cplx operator / (const Cplx& c1, const Cplx& c2)
{
    Cplx tmp(c1);
    return (tmp /= c2);
}

// overloaded operator for division of {\tt Cplx} / {\tt Cplx}
Cplx operator / (const Cplx& c, const Dbl& d)
{
    Cplx tmp(c);
    return (tmp /= d);
}

// overloaded operator for division of {\tt Cplx} / {\tt Cplx}
Cplx operator / (const Dbl& d, const Cplx& c)
{
    if (c.isInf()) return Cplx(0,Cplx::Nul);	// x / infinity = 0
    if (c.isNul()) return Cplx(0,Cplx::Inf);	// x / 0 = infinity
    return (d / c.value); 
}

//---------------------------------
// friend operators for comparison:
//---------------------------------

// overloaded operator == for comparison of two {\tt Cplx}
bool operator == (const Cplx &c1, const Cplx& c2)
{
    if (c1.isValid() && c2.isValid())
	return (c1.value == c2.value); 
    return (c1.isNul() && c2.isNul());
}

// overloaded operator != for comparison of two {\tt Cplx}
bool operator != (const Cplx &c1, const Cplx& c2)
{
    if (c1.isValid() && c2.isValid())
	return (c1.value != c2.value); 
    return !(c1.isNul() && c2.isNul());
}

// overloaded operator < for comparison of two {\tt Cplx}
bool operator < (const Cplx &c1, const Cplx& c2)
{
    if (c1.isValid() && c2.isValid())
	return (abs(c1.value) < abs(c2.value)); 
    if (c1.isNul() && c2.isNul()) return false;
    return (c1.isNul() && c2.isInf());
}

// overloaded operator > for comparison of two {\tt Cplx}
bool operator > (const Cplx &c1, const Cplx& c2)
{
    if (c1.isValid() && c2.isValid())
	return (abs(c1.value) > abs(c2.value)); 
    if (c1.isNul() && c2.isNul()) return false;
    return (c1.isInf() && c2.isNul());
}

// --- friend functions ---

END_NAMESPACE

