//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU Library General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING.LIB' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/Convert.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: Convert.c,v 1.5 2004/04/20 22:26:22 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/Convert.h>
USING_NAMESPACE(AH)

#include <cstdio>
USING_NAMESPACE(std)

// --- local stuff ---

#define TRACE 0

// --- class member implementation ---

BEGIN_NAMESPACE(AH)


// ----------------------------------------------------------------------
// convert big endian data into 32bit unsigned integer
// ----------------------------------------------------------------------
uint32 IOConvert::BigEndCharsToUint32(const uint8 data[])
{
    // we use fast multiplication: (X * 8) == (X << 3)
    uint32 val = 0;
    unsigned int idx = 0, dlen = sizeof(val);
    while (dlen--) val |= (data[idx++] << (dlen << 3));
    return val;
}


// ----------------------------------------------------------------------
// convert 32bit unsigned integer into big endian data
// ----------------------------------------------------------------------
void IOConvert::Uint32ToBigEndChars(uint32 val, uint8 data[])
{
    unsigned int dlen = sizeof(val);
    while (dlen--)
    {
	data[dlen] = (val & 0xff);
	val >>= 8;
    }
}


// ----------------------------------------------------------------------
// convert little endian data into 32bit unsigned integer
// ----------------------------------------------------------------------
uint32 IOConvert::LittleEndCharsToUint32(const uint8 data[])
{
    // we use fast multiplication: (X * 8) == (X << 3)
    uint32 val = 0;
    for (unsigned int idx = 0; idx < sizeof(val); ++idx)
	val |= (data[idx] << (idx << 3));
    return val;
}


// ----------------------------------------------------------------------
// convert 32bit unsigned integer into little endian data
// ----------------------------------------------------------------------
void IOConvert::Uint32ToLittleEndChars(uint32 val, uint8 data[])
{
    for (unsigned int idx = 0; idx < sizeof(val); ++idx)
    {
	data[idx] = (val & 0xff);
	val >>= 8;
    }
}


// ----------------------------------------------------------------------
// convert big endian data into 32bit signed integer
// ----------------------------------------------------------------------
int32 IOConvert::BigEndCharsToInt32(const uint8 data[])
{
    // we use fast multiplication: (X * 8) == (X << 3)
    int32 val = 0;
    unsigned int idx = 0, dlen = sizeof(val);
    while (dlen--) val |= (data[idx++] << (dlen << 3));
    return val;
}


// ----------------------------------------------------------------------
// convert 32bit signed integer into big endian data
// ----------------------------------------------------------------------
void IOConvert::Int32ToBigEndChars(int32 val, uint8 data[])
{
    unsigned int dlen = sizeof(val);
    while (dlen--)
    {
	data[dlen] = (val & 0xff);
	val >>= 8;
    }
}


// ----------------------------------------------------------------------
// convert little endian data into 32bit signed integer
// ----------------------------------------------------------------------
int32 IOConvert::LittleEndCharsToInt32(const uint8 data[])
{
    // we use fast multiplication: (X * 8) == (X << 3)
    int32 val = 0;
    for (unsigned int idx = 0; idx < sizeof(val); ++idx)
	val |= (data[idx] << (idx << 3));
    return val;
}


// ----------------------------------------------------------------------
// convert 32bit signed integer into little endian data
// ----------------------------------------------------------------------
void IOConvert::Int32ToLittleEndChars(int32 val, uint8 data[])
{
    for (unsigned int idx = 0; idx < sizeof(val); ++idx)
    {
	data[idx] = (val & 0xff);
	val >>= 8;
    }
}




// ----------------------------------------------------------------------
// convert big endian data into 16bit unsigned integer
// ----------------------------------------------------------------------
uint16 IOConvert::BigEndCharsToUint16(const uint8 data[])
{
    return (data[0] << 8) | data[1];
}


// ----------------------------------------------------------------------
// convert 16bit unsigned integer into big endian data
// ----------------------------------------------------------------------
void IOConvert::Uint16ToBigEndChars(uint16 val, uint8 data[])
{
    data[0] = (val >> 8) & 0xff;
    data[1] = val & 0xff;
}


// ----------------------------------------------------------------------
// convert little endian data into 16bit unsigned integer
// ----------------------------------------------------------------------
uint16 IOConvert::LittleEndCharsToUint16(const uint8 data[])
{
    return data[0] | (data[1] << 8);
}


// ----------------------------------------------------------------------
// convert 16bit unsigned integer into little endian data
// ----------------------------------------------------------------------
void IOConvert::Uint16ToLittleEndChars(uint16 val, uint8 data[])
{
    data[0] = val & 0xff;
    data[1] = (val >> 8) & 0xff;
}


// ----------------------------------------------------------------------
// convert big endian data into 16bit signed integer
// ----------------------------------------------------------------------
int16 IOConvert::BigEndCharsToInt16(const uint8 data[])
{
    return (data[0] << 8) | data[1]; 
}


// ----------------------------------------------------------------------
// convert 16bit signed integer into big endian data
// ----------------------------------------------------------------------
void IOConvert::Int16ToBigEndChars(int16 val, uint8 data[])
{
    data[0] = (val >> 8) & 0xff;
    data[1] = val & 0xff;
}


// ----------------------------------------------------------------------
// convert little endian data into 16bit signed integer
// ----------------------------------------------------------------------
int16 IOConvert::LittleEndCharsToInt16(const uint8 data[])
{
    return data[0] | (data[1] << 8);
}


// ----------------------------------------------------------------------
// convert 16bit signed integer into little endian data
// ----------------------------------------------------------------------
void IOConvert::Int16ToLittleEndChars(int16 val, uint8 data[])
{
    data[0] = val & 0xff;
    data[1] = (val >> 8) & 0xff;
}




// ----------------------------------------------------------------------
// convert time value into readable string
// ----------------------------------------------------------------------
string IOConvert::timeToString (unsigned int value, const char* unit)
{
    char buf[64];
    string retval;
    if (0 != strcmp(unit, "s"))
    {
	sprintf(buf, "%d ", value);
	retval = buf;
	retval += unit;
	return retval;
    }

    unsigned int seconds = value % 60;
    value /= 60;  // in minutes
    unsigned int minutes = value % 60;
    value /= 60;  // in hours
    unsigned int hours = value % 24;
    value /= 24;  // in days
    unsigned int days = value % 30;
    value /= 30;  // in months
    unsigned int months = value % 12;
    value /= 12;  // in years
    unsigned int years = value;

    if (years)
    {
	sprintf(buf, "%d year%s", years, (years > 1 ? "s":""));
	retval += buf;
    }
    if (months)
    {
	if (retval.size()) retval += " ";
	sprintf(buf, "%d month%s", months, (months > 1 ? "s":""));
	retval += buf;
    }
    if (days)
    {
	if (retval.size()) retval += " ";
	sprintf(buf, "%d day%s", days, (days > 1 ? "s":""));
	retval += buf;
    }
    if (retval.size())
    {
	if (!hours && !minutes && !seconds) return retval;
	retval += " ";
    }
    sprintf(buf, "%02d:%02d:%02d", hours, minutes, seconds);
    retval += buf;
    return retval;
}

// ----------------------------------------------------------------------
// convert time value into readable string
// ----------------------------------------------------------------------
string IOConvert::timeToString (double value, const char* unit)
{
    char buf[64];
    string retval;
    if (0 != strcmp(unit, "s"))
    {
	sprintf(buf, "%f ", value);
	retval = buf;
	retval += unit;
	return retval;
    }

    unsigned int ival = (unsigned int)value;
    double remval = value - (double)ival;
    if (1 < remval)
    {
	// value is too big for an integer
	sprintf(buf, "%f ", value);
	retval = buf;
	retval += unit;
	return retval;
    }
    if (ival) 
    {
	// more than 1 second:
	retval = timeToString(ival, unit);
    }
    else retval = "0";

    //--------------------------------
    // calculate the fractional part:
    //--------------------------------
    sprintf(buf, "%09d", (int)(remval *1e9 + 0.5));
    bool ns = true;
    bool us = true;
    bool ms = true;
    if (0 == strcmp(buf+6, "000"))
    {
	ns = false;
	sprintf(buf, "%06d", (int)(remval *1e6 + 0.5));
	if (0 == strcmp(buf+3, "000"))
	{
	    us = false;
	    sprintf(buf, "%03d", (int)(remval *1e3 + 0.5));
	    if (0 == strcmp(buf, "000"))
		ms = false;
	}
    }

    if (ns)
	sprintf(buf, ".%09d", (int)(remval *1e9 + 0.5));
    else if (us)
	sprintf(buf, ".%06d", (int)(remval *1e6 + 0.5));
    else if (ms)
	sprintf(buf, ".%03d", (int)(remval *1e3 + 0.5));
    else
	return retval;  // no fractional time
    if (!isdigit(retval[retval.size()-1]))
	retval += " 0"; // just days or more
    retval += buf;

    return retval;
}


// ----------------------------------------------------------------------
// convert time value in seconds into readable string
// ----------------------------------------------------------------------
string IOConvert::timeToCdString (double value)
{
    string retval;
    char buf[80];
    unsigned int ival = (unsigned int)value;
    double frac_sec = value - (double)ival;
    if (1 < frac_sec)
    {
	// value is too big for an integer
	sprintf(buf, "%f ", value);
	retval = buf;
	return retval;
    }

    const double eps = 0.5 * 75.0 / 44100.0;

    unsigned int seconds = ival % 60;
    ival /= 60;  // in minutes
    unsigned int minutes = ival % 60;
	
    frac_sec *= 75;         // in CDDA sectors (75 sectors per second)
    unsigned int sectors = (int)(frac_sec + eps);
    double rem_sect = frac_sec - (double)sectors;

    if (rem_sect > eps)
	sprintf(buf, "%02d:%02d:%05.3f", minutes, seconds, frac_sec);
    else
	sprintf(buf, "%02d:%02d:%02d", minutes, seconds, sectors);
    retval = buf;
    return retval;
}


END_NAMESPACE













