
#include <vector>
#include <string>

class IOCmdOption
{
 public:
    enum SecondArg 
    {
	/** no second argument. */
	SecondArgNone, 
	/** needs a second argument. */
	SecondArgAlways,
	/** may have a second argument. */
	SecondArgOptional
    };

    // in client: override if necessary in subclasses ----------

    /** constructor. */
    IOCmdOption(char short_opt, const char* long_opt, SecondArg sec,
		const std::string& desc)
	: shortOpt(short_opt), longOpt(long_opt), secArg(sec),
	description(desc) {}

    /** destructor */
    virtual ~IOCmdOption();

    // in client: override if necessary, do not use -------------

    /** set option to set. */
    virtual void set()
	{ ++first; }

    /** set second argument. */
    virtual void setSecArg(const std::string& arg)
	{ second.push_back(arg); }

    // used by client only: -------------------------------------

    /** return true if option is set. */
    int isSet() const 
	{ return first; }

    /** return second argument. */
    std::vector<std::string> getSecArg() const
	{ return second; }

    // used by IOCmdOptHandler only: ----------------------------

    /** return true if it is a short option. */
    bool isShort() const
	{ return (shortOpt != '\0'); }

    /** return true if it is a long option. */
    bool isLong() const
	{ return (longOpt != 0); }

    /** return second argument. */
    SecondArg getSecArgMode() const
	{ return secArg; }

    /** return short option char. */
    char getShortOpt() const
	{ return shortOpt; }

    /** return long option name. */
    const char* getLongOpt() const
	{ return longOpt; }
   
 protected:
    char        shortOpt;
    const char* longOpt;
    SecondArg   secArg;
    std::string description;

 private:
    int first;
    std::vector<std::string> second;
};	// IOCmdOption

class dummys 
{
 public:
    dummys() {}
    void add(IOCmdOption* dum) {a.push_back(dum); }
    std::vector<IOCmdOption*> a;
};

class dummys2
{
 public:
    dummys2() {}
    void add2(IOCmdOption* dum) { b.push_back(dum); }
    void add (IOCmdOption* dum) { a.push_back(dum); }
    std::vector<IOCmdOption*> b;
    std::vector<IOCmdOption*> a;
};


void test()
{
    IOCmdOption* x = 0;
    dummys d;
    d.add(x);
}
