const char sweep_CVSID[] = "$Id: FtWisdom.c,v 1.2 2004/04/20 22:34:39 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <AH/Math/Ft/Wisdom.h>
#include <drfftw.h> 
#include <cerrno> 
#include <sys/types.h>
#include <sys/stat.h>
#include <string> 
#include <iostream>

#include <AH/Error/Basic.h>

// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

MathFtWisdom::MathFtWisdom(unsigned int debug)
    : fileName("/usr/share/fftw/wisdom_r"), 
      debon(debug)
{
    // import wisdom
    this->importFromFile();
}


MathFtWisdom::~MathFtWisdom()
{
    // destroy existing plan
    for (unsigned int i = 0; i < table.size(); ++i)
	fftw_destroy_plan(table[i].plan);

    // export wisdom
    this->exportToFile();
}


void MathFtWisdom::importFromFile()
{
    // import wisdom
    FILE* fp = fopen(fileName.c_str(), "r");
    if (fp)
    {
	if (debon > 2)
	    std::cerr << "FtWisdom::importFromFile: import wisdom" << std::endl;
	fftw_import_wisdom_from_file(fp);
	fclose(fp);
    }
    else std::cerr << "Error reading " << fileName << ": "
	      << strerror(errno) << std::endl;
}


void MathFtWisdom::exportToFile()
{
    // make wisdom accessible for all users:
    mode_t curmask = umask(0);

    // export wisdom
    FILE* fp = fopen(fileName.c_str(), "w");
    if (fp)
    {
	if (debon > 2)
	    std::cerr << "FtWisdom::exportToFile: export wisdom" << std::endl;
	fftw_export_wisdom_to_file(fp);
	fclose(fp);
    }
    else std::cerr << "Error creating " << fileName << ": "
	      << strerror(errno) << std::endl;

    umask(curmask);
}



rfftw_plan MathFtWisdom::getPlan(unsigned int num, fftw_direction direction)
{
    // try to find a matching plan in our table:
    for (unsigned int i = 0; i < table.size(); ++i)
    {
	if ((num == table[i].size) &&
	    (direction == table[i].dir)) 
	    return table[i].plan;
    }

     /* create a plan for N=num, possibly creating and/or using wisdom */
    if (debon > 1) 
	std::cerr << "FtWisdom::getPlan: create plan for N=" << num << std::endl;
    rfftw_plan plan = rfftw_create_plan(num, direction,
					FFTW_MEASURE | FFTW_USE_WISDOM);
    if (0 == plan) throw ErrorBasic("Cannot create FFTW plan");
    if (debon > 2)
	std::cerr << "FtWisdom::getPlan: plan for N=" << num << " created" << std::endl;
//    this->exportToFile(); // just in case

    // add new plan into our table
    Entry new_ent;
    new_ent.size = num;
    new_ent.dir = direction;
    new_ent.plan = plan;
    table.push_back(new_ent);

    return new_ent.plan;
}


// --- end of file ---
