//---------------------------------------------------------------------------
// Copyright 2005 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHmath/testMathDither.c,v $
// $Id: testMathDither.c,v 1.1 2005/02/06 13:40:47 ah Exp $
//
// Description: test PDF (probability distribution function) of MathDither
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: testMathDither.c,v 1.1 2005/02/06 13:40:47 ah Exp $ (c) 2005 Andreas Huennebeck";

// --- necessary includes ---

#include <iostream>
#include <fstream>
#include <AH/Math/Dither.h>

// --- global function implementation ---


int main()
{
    std::cerr << "Testprogram for class MathDither in libMath" << std::endl;

    //---------------------------------------------------------------------

    MathDither r(MathDither::RectAngle);
    MathDither t(MathDither::Triangle);

    const unsigned int MAX_BOX = 100;
    const unsigned int tries = MAX_BOX * 10000;

    // arrays for PDF:
    unsigned int rbox[MAX_BOX+1];
    unsigned int tbox[MAX_BOX+1];
    for (unsigned int i = 0; i < MAX_BOX+1; ++i)
    {
	rbox[i] = 0;
	tbox[i] = 0;
    }

    for (unsigned int i = 0; i < tries; ++i)
    {
	// get dither values:
	double dr = r.getSignal();
	double dt = t.getSignal();

	// put them into the correct box:
	// get index into array from value:

	// rectangular: range is -0.5 .. +0.5 ==> box = 0..MAX_BOX
	// idx = (val + 0.5) * MAX_BOX
	dr += 0.5; // range is now 0 .. +1
	dr *= MAX_BOX; // range now 0..MAX_BOX
	unsigned int ir = (unsigned int)(dr + 0.5);
	if ((ir >= 0) && (ir <= MAX_BOX))
	    rbox[ir] += 1;

	// triangular: range is -1 .. +1 ==> box = 0..MAX_BOX
	// idx = (val + 1) * MAX_BOX/2
	dt += 1; // range is now 0 .. +2
	dt *= MAX_BOX / 2.0; // range now 0..MAX_BOX
	unsigned int it = (unsigned int)(dt + 0.5);
	if ((it >= 0) && (it <= MAX_BOX))
	    tbox[it] += 1;
    } 

    unsigned int rmax = 0;
    unsigned int tmax = 0;
    for (unsigned int i = 0; i < MAX_BOX+1; ++i)
    {
	rmax += rbox[i];
	tmax += tbox[i];
    }
    std::cerr << "reactangular dither: caught " << rmax << " from " 
	      << tries << " tries.\n";
    std::cerr << "triangular dither: caught " << tmax << " from " 
	      << tries << " tries.\n";

    std::ofstream pdf("tmd.dat");
    pdf << "rectangular PDF | triangular PDF\n";
    for (unsigned int i = 0; i < MAX_BOX+1; ++i)
    {
	// calculate PDF:
	double pr = (double)rbox[i] / (double)rmax;
	// get value from index into array:
	// val = idx/MAX_BOX - 0.5
	double xr = (double)i / (double)MAX_BOX - 0.5;

	// calculate PDF:
	double pt = (double)tbox[i] / (double)tmax;
	// get value from index into array:
	// val = 2*idx/MAX_BOX - 1
	double xt = 2.0 * (double)i / (double)MAX_BOX - 1.0;

	pdf << xr << " " << pr << "   " << xt << " " << pt << "\n";
    }

    return (0);
}




