//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/wavcut.c,v $
// $Id: wavcut.c,v 1.7 2000/11/24 20:50:28 ah Exp $ 
//
// Description: cut a pice out of a wav file into a new wav file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: wavcut.c,v 1.7 2000/11/24 20:50:28 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <stdlib.h>
#ifdef NO_UNISTD
#   include <getopt.h>
#else
#   include <unistd.h>
#endif
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
USING_NAMESPACE(std)

#include <AH/Error/Class.h>
#include <AH/Error/Basic.h>
#include <AH/IO/Riff/WaveFile.h>
USING_NAMESPACE(AH)

#include "filecopy.h"

// --- local constants ---

// --- local variables ---

static int verbose = 0;

// --- local classes ---

// --- local functions ---


//----------------------------------------------------------------
// check input files for existence and for matching audio format
// On Error: throws an exception
//----------------------------------------------------------------
static void read_one_file
(
    const FileSettings& files,
    int fileNum,
    IORiffWaveFile& out,
    const string& outputFileName,
    unsigned char* buf,
    uint32 bufSize
)
{
    // create a wav file object:
    const string& inputFileName = files.getFileName(fileNum);
    if (3 < verbose) cerr << "open input file \"" << inputFileName
			  << "\":" << endl;
    IORiffWaveFile in(inputFileName.c_str());
    if (verbose  > 3) in.printInfo(cerr);

    // get desired range (samples or times):
    bool untilEnd = true;
    uint32 endPos;

    uint32 samNum = in.getSampleNum();
    int s1, s2;
    files.getSampleRange(fileNum, s1, s2);
    double t1, t2;
    files.getTimeRange(fileNum, t1, t2);
    if ((0 <= t1) || (0 <= t2))
    {
	// time setting overrides sample setting:
	if (0 <= t1)
	{
	    in.setDataPos(t1);
	    samNum -= in.getSampleNum(t1);
	}
	else in.setDataPos((uint32)0);
	if (0 <= t2)
	{
	    untilEnd = false;
	    endPos = in.getDataPos(t2);
	    samNum -= in.getSampleNum() - in.getSampleNum(t2);
	}
	else
	    endPos = in.getDataPos(in.getSampleNum());
    }
    else
    {
	// use sample setting:
	if (0 <= s1)
	{
	    in.setDataPos((uint32)s1);
	    samNum -= s1;
	}
	else in.setDataPos((uint32)0);
	if (0 <= s2)
	{
	    untilEnd = false;
	    endPos = in.getDataPos((uint32)s2);
	    samNum -= in.getSampleNum() - s2;
	}
	else
	    endPos = in.getDataPos(in.getSampleNum());
    }
    if (verbose > 4)
	cerr << "read_one_file: Taking " << samNum << " samples" << endl;

    // read desired data range:
    uint32 sizeToRead = endPos - in.getDataPos();
    do
    {
	// how much can we read (not more than buffer size) ?
	uint32 curBufSize = sizeToRead;
	if (curBufSize > bufSize) curBufSize = bufSize;

	// read input file data :
	if (4 < verbose) cerr << "read " << curBufSize 
			      << " bytes from input file \"" 
			      << inputFileName << "\":" << endl;
	int32 rdsize = in.readRaw(buf, curBufSize);
	if (0 >= rdsize) break;
	sizeToRead -= rdsize;

	// write output file data :
	uint32 wrsize;
	while (0 < rdsize)
	{
	    if (4 < verbose) cerr << "write " << rdsize 
				  << " bytes into output file \"" 
				  << outputFileName << "\":" << endl;
	    wrsize = out.writeRaw(buf, rdsize);
	    if (0 >= wrsize) break;
	    rdsize -= wrsize;
	}
	if (0 >= wrsize) break;
    }
    while (sizeToRead);
} // end of read_one_file()


// --- end of file ---
