//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/reduce.c,v $
// $Id: reduce.c,v 1.2 2005/02/17 21:03:09 ah Exp $ 
//
// Description: create frequency spectrum of a wav file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: reduce.c,v 1.2 2005/02/17 21:03:09 ah Exp $ (c) 2004 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <drfftw.h> 
USING_NAMESPACE(std)

#include <AH/IO/Smart/Buffer.h>
#include "reduce.h"
USING_NAMESPACE(AH)

// --- local definitions ---

// --- local constants ---

// --- local variables ---

// --- local classes ---

// --- local functions ---

// --- member functions ---

// keep linear frequency scale, reduce output size.
void ReduceFFTW::processLinXScale(fftw_real* data, unsigned int insize, 
			    double* avout, double* pkout, unsigned int outsize,
			    double f_max, double f1, double f2)
{
    if (!avout && !pkout) return;
    if (!insize || !outsize) return;
    if ((f1 < 0) || (f2 <= 0) || (f_max <= 0))
	throw ErrorInternBadArg("ReduceFFTW::processLinXScale");

    if (this->verbose > 1) 
	cerr << "ReduceFFTW::processLinXScale: make lin scale for frequency" 
	     << endl;

    const double fin_inc = f_max/(double)(insize - 1);
    double fout = f1;
    const double fout_inc = (f2 - f1) / (outsize - 1.0);
    const double flim_inc = 0.5 * fout_inc;

    for (unsigned int out = 0; out < outsize; ++out, fout += fout_inc)
    {
	// determine index span (float) of this output frequency:
	double fi1 = (fout - flim_inc) / fin_inc;
	double fi2 = (fout + flim_inc) / fin_inc;

	// determine first and last index:
	int bi1 = (int)(fi1 + 0.5);
	int bi2 = (int)(fi2 + 0.5);

	// determine proportional parts of first and last index:
	double pi1; // proportional part of bi1
	double pi2; // proportional part of bi2
	if (bi1 == bi2)
	{
	    pi1 = 0.5 * fabs(fi2 - fi1);
	    pi2 = pi1;
	}
	else
	{
	    pi1 = (double)bi1 + 0.5 - fi1;
	    pi2 = fabs(fi2 - (double)bi2 + 0.5);
	}

	// sum up over all indices:
	double max = data[bi1] > data[bi2] ? data[bi1] : data[bi2];
	double sum = pi1 * data[bi1] + pi2 * data[bi2];
	double cnt = pi1 + pi2;
	for (int i = bi1+1; i < bi2; ++i)
	{
	    if (data[i] > max) max = data[i];
	    sum += data[i];
	    ++cnt;
	}
	if (avout) avout[out] = sum/cnt;
	if (pkout) pkout[out] = max;
    }

} // end of ReduceFFTW::processLinXScale()



// convert linear frequency scale into log scale, thereby reducing
// output size.
void ReduceFFTW::processLogXScale(fftw_real* data, unsigned int insize, 
			    double* avout, double* pkout, unsigned int outsize,
			    double f_max, double f1, double f2, double smooth)
{
    if (!avout && !pkout) return;
    if (!insize || !outsize) return;
    if ((f1 <= 0) || (f2 <= 0) || (f_max <= 0))
	throw ErrorInternBadArg("ReduceFFTW::processLogXScale");

    if (this->verbose > 1) 
	cerr << "ReduceFFTW::processLogXScale: make log scale for frequency" 
	     << endl;

    const double fin_inc = f_max/(double)(insize - 1);
    double fout = f1;
    const double fout_inc = pow(f2/f1, 1.0 / (outsize - 1.0));
    const double flim_inc = pow(f2/f1, 0.5 / (outsize - 1.0));

    for (unsigned int out = 0; out < outsize; ++out, fout *= fout_inc)
    {
	// determine frequency span of this output frequency:
	double fs1 = fout / flim_inc;
	double fs2 = fout * flim_inc;
	if (fs1 < 0) fs1 = 0;
	if (fs2 > f_max) fs2 = f_max;

	// determine index span (float) of this output frequency:
	double fi1 = fs1 / fin_inc;
	double fi2 = fs2 / fin_inc;

	// determine first and last index:
	int bi1 = (int)(fi1 + 0.5);
	int bi2 = (int)(fi2 + 0.5);
	if (bi1 < 0) bi1 = 0;
	if (bi2 >= (int)insize) bi1 = insize - 1;

	// determine proportional parts of first and last index:
	double pi1; // proportional part of bi1
	double pi2; // proportional part of bi2
	if (bi1 == bi2)
	{
	    pi1 = 0.5 * fabs(fi2 - fi1);
	    pi2 = pi1;
	}
	else
	{
	    pi1 = (double)bi1 + 0.5 - fi1;
	    pi2 = fabs(fi2 - (double)bi2 + 0.5);
	}

	// sum up over all indices:
	double max = data[bi1] > data[bi2] ? data[bi1] : data[bi2];
	double sum = pi1 * data[bi1] + pi2 * data[bi2];
	double cnt = pi1 + pi2;
	for (int i = bi1+1; i < bi2; ++i)
	{
	    sum += data[i];
	    if (data[i] > max) max = data[i];
	    ++cnt;
	}
	// I do not really understand why we must divide the sum
	// by sqrt(cnt), but if we don't, then a pink noise spectrum
	// rises by 10 dB per octave. 
	if (avout) avout[out] = sum / sqrt(cnt);
	if (pkout) pkout[out] = max;
    }

    if (avout && (smooth > 0))
	this->smoothLogXScale(avout, outsize, outsize, smooth, f1, f2);

} // end of ReduceFFTW::processLogXScale()



// smooth data in logarithmic frequency scale
void ReduceFFTW::smoothLogXScale(double* data, unsigned int insize, 
			      unsigned int outsize, double smooth,
			      double f1, double f2)
{
    if (this->verbose > 1) 
	cerr << "ReduceFFTW::smoothLogXScale: smooth log scale for frequency" 
	     << endl;

    IOSmartBuffer<double> dout(outsize);

    // convert number of octaves into range factor:
    // (a) fs2 = 2^smooth * fs1
    // (b) sqrt(fs1 * fs2) = f
    // ==> fs1 = f / sqrt(2^smooth) and fs2 = f * sqrt(2^smooth)
    double range = sqrt(pow(2, smooth));

    double f = f1;
    const double f_inc = pow(f2/f1, 1.0 / (outsize - 1.0));
    const double log_f_inc = log(f_inc);
    for (unsigned int out = 0; out < outsize; ++out, f *= f_inc)
    {
	// determine frequency span of smoothing range:
	double fs1 = f / range;
	double fs2 = f * range;
	if (fs1 < f1) fs1 = f1;
	if (fs2 > f2) fs2 = f2;

	// determine index span (float) of smoothing range:
	double fi1 = log(fs1/f1) / log_f_inc;
	double fi2 = log(fs2/f1) / log_f_inc;

	// determine first and last index:
	int bi1 = (int)(fi1 + 0.5);
	int bi2 = (int)(fi2 + 0.5);
	if (bi1 < 0) bi1 = 0;
	if (bi2 >= (int)outsize) bi1 = outsize - 1;

	// determine proportional parts of first and last index:
	double pi1; // proportional part of bi1
	double pi2; // proportional part of bi2
	if (bi1 == bi2)
	{
	    pi1 = 0.5 * fabs(fi2 - fi1);
	    pi2 = pi1;
	}
	else
	{
	    pi1 = (double)bi1 + 0.5 - fi1;
	    pi2 = fabs(fi2 - (double)bi2 + 0.5);
	}

	// sum up over all indices:
	double sum = pi1 * data[bi1] + pi2 * data[bi2];
	double cnt = pi1 + pi2;
	for (int i = bi1+1; i < bi2; ++i)
	{
	    sum += data[i];
	    ++cnt;
	}
	dout[out] = sum / cnt;
    }

    // copy result back into input buffer, update input buffer size
    for (unsigned int i = 0; i < outsize; ++i)
    {
	data[i] = dout[i];
    }

} // end of ReduceFFTW::smoothLogXScale()


#if 0
// convert amplitude into dB
void make_ylog_scale(double* data, unsigned int size,
		     double maxamp, double minval, double mindb)
{
    if (verbose > 1) cerr << "convert amplitude into dB" << endl;

    // convert absolute values into dB
    for (unsigned int i = 0; i < size; ++i)
    {
	double val = data[i];
	data[i] = (val < minval ? mindb : 20 * log10(val/maxamp));
	if (verbose > 3) cerr << "lin = " << val 
			      << " log = " << data[i] << endl;
    }
} // make_ylog_scale()
#endif


// --- global functions ---




// --- end of file ---
